@@ -1,6 +1,5 @@
 /*
  * Mesa 3-D graphics library
- * Version:  7.11
  *
  * Copyright (C) 2011 Benjamin Franzke <benjaminfranzke@googlemail.com>
  *
@@ -163,7 +162,7 @@ wayland_shm_display_init_screen(struct native_display *ndpy)
       return FALSE;
 
    if (shmdpy->base.formats == 0)
-      wl_display_roundtrip(shmdpy->base.dpy);
+      wayland_roundtrip(&shmdpy->base);
    if (shmdpy->base.formats == 0)
       return FALSE;
 
