@@ -58,10 +58,7 @@ struct radeon_drm_winsys {
     uint32_t accel_working2;
 
     struct pb_manager *kman;
-    struct pb_manager *cman_vram;
-    struct pb_manager *cman_vram_gtt_wc;
-    struct pb_manager *cman_gtt;
-    struct pb_manager *cman_gtt_wc;
+    struct pb_manager *cman;
     struct radeon_surface_manager *surf_man;
 
     uint32_t num_cpus;      /* Number of CPUs. */
