@@ -39,9 +39,9 @@ upload_sampler_state_pointers(struct brw_context *brw)
 	     GS_SAMPLER_STATE_CHANGE |
 	     PS_SAMPLER_STATE_CHANGE |
 	     (4 - 2));
-   OUT_BATCH(brw->sampler.offset); /* VS */
-   OUT_BATCH(0); /* GS */
-   OUT_BATCH(brw->sampler.offset);
+   OUT_BATCH(brw->vs.base.sampler_offset); /* VS */
+   OUT_BATCH(brw->gs.base.sampler_offset); /* GS */
+   OUT_BATCH(brw->wm.base.sampler_offset);
    ADVANCE_BATCH();
 }
 
